(in-package :xml-psychiatrist)

(defclass attr-matcher (matcher)
  ((possible-values :initarg :possible-values)
   (type :initarg :type)
   (matches-regexp :initarg :matches-regexp)
   (test-fn :initarg :test-fn))
  (:documentation "A matcher for an attribute"))

(defmethod match ((matcher attr-matcher) node)
  (with-slots (possible-values type matches-regexp test-fn name) matcher
     (and (t-unless possible-values
		    (member (second node)
			    possible-values
			    :test #'string=))
			     
	  (t-unless type
		    (is-type-p (second node)
			       type))
	  (t-unless matches-regexp
		    (let ((regexp (concatenate 'string
					       "^"
					       matches-regexp
					       "$")))
		      (cl-ppcre:scan regexp
				     (second node))))
	  (t-unless test-fn
		    (funcall test-fn (second node)))
	  (name-matches-p (first node)
			  name))))

(defun attr (name &key (how-many :one-or-more) possible-values type matches-regexp test-fn)
  "Return a matcher for an attribute. Certain constraints are possible, such as :possible-values '(val1 val2 val3 ...)"
  (make-instance 'attr-matcher
		 :possible-values possible-values
		 :type type
		 :matches-regexp matches-regexp
		 :test-fn test-fn
		 :how-many how-many
		 :name name))