(in-package :xml-psychiatrist)

(defclass matcher ()
  ((how-many :initarg :how-many
	     :reader how-many
	     :documentation "One of `:one', `:one-or-more', `:zero-or-more', `:zero-or-one'")
   (name :initarg :name
	 :reader name))
  (:documentation "An object that matches some part of an XML tree. It has a matcher function and can be declared optional.
It also has a name, for use in introspection and error reporting. The name is the same as the name of the XML node that
the matcher matches; for example, a matcher for a `<foo>' tag would have a `name' of `foo'."))

(defgeneric match (matcher node)
  (:documentation "Does matcher match node?"))

(defun toplevel-match (matcher node)
  "Top-level equivalent of `match'. This is what users should call."
  (let ((result (catch 'match-exit
		  (match matcher node))))
    (apply #'values (cond ((null result)
			   (list nil
				 (format nil "Matcher ~S did not match"
					 matcher)))
			  ((listp result)
			   result)
			  (result (list result nil))))))