(in-package :xml-psychiatrist)

;; The PCDATA matcher
;;;;;;;;;;;;;;;;;;;;;

(defclass pcdata-matcher (matcher)
  ((possible-values :initarg :possible-values)
   (matches-regexp :initarg :matches-regexp)
   (type :initarg :type)
   (test-fn :initarg :test-fn)))

(defmethod match ((matcher pcdata-matcher) node)
  (with-slots (possible-values type matches-regexp test-fn) matcher
     (and (stringp node)
	  (t-unless test-fn
		    (funcall test-fn node))
	  (t-unless matches-regexp
		    (cl-ppcre:scan matches-regexp
				   node))
	  (t-unless type
		    (is-type-p node
			       type))
	  (t-unless possible-values
		    (member node
			    possible-values
			    :test #'string=)))))

(defun pcdata (&key matches-regexp (how-many :one-or-more) test-fn possible-values type)
  "Matches only PCDATA---that is, strings. It can be constrained with:

:match-regexp `regexp'
:test-fn `function'
:possible-values `values-list'
:type `type'"
  (make-instance 'pcdata-matcher
		 :possible-values possible-values
		 :type type
		 :matches-regexp matches-regexp
		 :test-fn test-fn
		 :how-many how-many
		 :name "PCDATA"))

;; fun-matchers -- matchers that call a function
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;

(defclass fun-matcher (matcher)
  ((fun :initarg :fun
	:accessor fun))
  (:documentation "A matcher which calls a function `fun' on a node"))

(defmethod match ((matcher fun-matcher) node)
  (funcall (fun matcher)
	   node))

;; Match anything
;;;;;;;;;;;;;;;;;

(defun match-anything (&key (how-many :one-or-more))
  "Matches anything."
  (make-instance 'fun-matcher
		 :fun #'(lambda (node)
			  (declare (ignore node))
			  t)
		 :how-many how-many
		 :name "ANYTHING"))