(require 'asdf)
(in-package :xml-psychiatrist)
(asdf:oos 'asdf:load-op :fiveam)
(use-package :5am)

(def-suite validate-suite
  :description "XML validation test suite")

(in-suite validate-suite)

(defun string-equals-foo (string)
 "Only match the string 'Foo'"
 (string= string
          "Foo"))

(test attribute-validation
      "Test the validation of attributes"
      (is-true (match (attr "foo")
		      '("foo" "bar"))
	       "Attributes do not match what they should"))

(let ((person-tag (tag "person" ((attr "lastname"
				       :how-many :zero-or-more
				       :matches-regexp "[a-z]+")
				 (attr "firstname"))
		       (pcdata)))
      (person-wihtout-attrs-tag (tag "person" ()
				     (pcdata)))
      (real-person-tag (tag "person" ((attr "gender" :possible-values '("male" "female")))
			    (tag* "attribute" ()
				  (pcdata))
			    (tag "name" ()
				 (tag "last" ()
				      (pcdata))
				 (tag "first" ()
				      (pcdata)))))
      (fellow-tag (tag "fellow" ((attr "age"
				       :type '(integer 30 50)))
		       (pcdata)))
      (regexp-tag (tag "p[ea]rson" ((attr "lastname"
					  :how-many :zero-or-more)
				    (attr "firstname"))
		       (pcdata)))
      (number-tag (tag "number" ()
		       (pcdata :matches-regexp "^[0-9]+$")))
      (lowercase-person-tag (tag "person" ((attr "lastname"
						 :how-many :zero-or-more
						 :matches-regexp "[a-z]+")
					   (attr "firstname"
						 :matches-regexp "[a-z]+"))
				 (pcdata)))
      (foo-attr-tag (tag "person" ((attr "str"
					 :test-fn #'string-equals-foo))
			 (pcdata)))
      (foo-name-tag (tag #'string-equals-foo ()
			 (pcdata)))
      (foo-pcdata-tag (tag "foo" ()
			 (pcdata :test-fn #'string-equals-foo)))
      (namespaced-attr-tag (tag "foo" ((attr '("person" . "http\:\/\/authors")))
				(pcdata))))
  (test tag-validation
	"Test the tag validation"
	(is-true (toplevel-match person-tag
				      '("person" (("lastname" "scott") ("firstname" "peter")) "Hello"))
		 "Basic testing failed")
	(is-false (toplevel-match person-tag
				  '("person" (("lastname" "scOtt") ("firstname" "peter")) "Hello"))
		  "Optional attribute regexp matching failed")
	(is-false (toplevel-match person-tag
				       '("persono" (("lastname" "scott") ("firstname" "peter")) "Hello"))
		  "Tag-name matching too generous")
	(is-true (toplevel-match person-tag
				      '("person" (("firstname" "peter")) "Hello"))
		 "Optional attributes are not treated as such")
	(is-false (toplevel-match person-tag
				       '("person" (("lastname" "peter")) "Hello"))
		  "Non-optional tags are being treated as optional")
	(is-true (toplevel-match person-wihtout-attrs-tag
				      '("person" () "Hello"))
		 "Tags without attrs do not match matching elements without attrs")
	(is-false (toplevel-match person-wihtout-attrs-tag
				       '("person" (("foo" "bar")) "Hello"))
		  "Extra tags are being ignored")
	(is-true (toplevel-match real-person-tag
				      '("person" (("gender" "male"))
					("name" NIL ("first" NIL "Peter") ("last" NIL "Scott"))
					("attribute" NIL "Frozen")
					("attribute" NIL "Pleasant"))))
	(is-false (toplevel-match real-person-tag
				       '("person" (("gender" "neuter"))
					 ("name" NIL ("first" NIL "Peter") ("last" NIL "Scott"))
					 ("attribute" NIL "Frozen")
					 ("attribute" NIL "Pleasant")))
		  "Attribute :possible-values constraints are not working")
	(is-true (toplevel-match fellow-tag
				 '("fellow" (("age" "42")) "Hello"))
		 "Integer types improperly being rejected")
	(is-false (toplevel-match fellow-tag
				  '("fellow" (("age" "25")) "Hello"))
		  "Out-of-range integers are being erroneously accepted")
	(is-true (toplevel-match regexp-tag
				 '("parson" (("lastname" "scott") ("firstname" "peter")) "Hello"))
		 "Regexps aren't matching")
	(is-true (toplevel-match number-tag
				 '("number" () "13"))
		 "PCDATA regexp malfunction (false negative)")
	(is-false (toplevel-match number-tag
				  '("number" () "not a number"))
		 "PCDATA regexp malfunction (false positive)")
	(is-true (toplevel-match lowercase-person-tag
				 '("person" (("lastname" "scott") ("firstname" "peter")) "Hello"))
		 "Attr regexp matching malfunction (false negative)")
	(is-false (toplevel-match lowercase-person-tag
				  '("person" (("lastname" "Scott") ("firstname" "Peter")) "Hello"))
		  "Attr regexp matching malfunction (false positive)")
	(is-true (toplevel-match foo-attr-tag
				 '("person" (("str" "Foo")) "Hello"))
		 "Attribute test function malfunction (false negative)")
	(is-false (toplevel-match foo-attr-tag
				  '("person" (("str" "something other than Foo")) "Hello"))
		  "Attribute test function malfunction (false positive)")
	(is-true (toplevel-match foo-name-tag
				 '("Foo" () "Hello"))
		 "Name test function malfunction (false negative)")
	(is-false (toplevel-match foo-name-tag
				  '("Something other than Foo" () "Hello"))
		  "Name test function malfunction (false positive)")
	(is-true (toplevel-match namespaced-attr-tag
				 '("foo" ((("person" . "http://authors")
					   "Peter"))
				   "here there be pcdata"))
		 "Namespaced attribute malfunction (false negative)")
	(is-false (toplevel-match namespaced-attr-tag
				  '("foo" (("person"
					    "Peter"))
				    "here there be pcdata"))
		  "Namespaced attribute malfunction (false positive)")
	(is-true (toplevel-match foo-pcdata-tag
				 '("foo" ()
				   "Foo"))
		 "PCDATA test function malfunction (false negative)")
	(is-false (toplevel-match foo-pcdata-tag
				  '("foo" ()
				    "not Foo"))
		  "PCDATA test function malfunction (false positive)")))

(test name-matches-p
      "Test name-matches-p"
      (is-true (name-matches-p "foo" "f.o"))
      (is-false (name-matches-p "fo" "f.o"))
      (is-true (name-matches-p '("foo" . "bar") '("f.o" . "bar")))
      (is-false (name-matches-p '("foo" . "bzr") '("f.o" . "bar")))
      (is-false (name-matches-p '("fool" . "bar") '("f.o" . "bar")))
      (is-false (name-matches-p "fo" 'string-equals-foo))
      (is-true (name-matches-p "Foo" 'string-equals-foo)))

;(run! 'validate-suite)