(in-package :xml-psychiatrist)

(defun xml-validation-error (format-string &rest arguments)
  "Write an XML validation error to the appropriate stream, using the format syntax with specified arguments."
  (throw 'match-exit
	 (list nil
	       (apply #'format nil
		      format-string arguments))))

(defun safe-node-name (node)
  "Safe version of node-name"
  (if (stringp node)
      "PCDATA"
    (node-name node)))

(defun read-safely-from-string (str)
  "Safe version of read-from-string"
  (let ((*read-eval* nil)
        (*error-output* (make-broadcast-stream)))
    (ignore-errors (read-from-string str :preserve-whitespace t))))

(defun is-type-p (string type)
  "Is the datum in `string' (readable) of type `type'?"
  (let ((datum (read-safely-from-string string)))
    (typep datum
	   type)))

(defun name-matches-p (name name-regexp)
  (typecase name-regexp
     (string
      (cl-ppcre:scan (concatenate 'string
				  "^"
				  name-regexp
				  "$")
		     name))
     (cons
      (and (consp name)
	   (cl-ppcre:scan (concatenate 'string
				       "^"
				       (car name-regexp)
				       "$")
			  (car name))
	   (cl-ppcre:scan (concatenate 'string
				       "^"
				       (cdr name-regexp)
				       "$")
			  (cdr name))))
     ((or symbol function)
      (funcall name-regexp name))))

(defmacro t-unless (predicate &body body)
  "Return t unless predicate is true, in which case `body' will be evaluated
in an implicit progn to get the return value."
  `(if ,predicate
       (progn ,@body)
     t))